@@ -16,8 +16,10 @@
 #' @param allow.skips (To be replaced with additional options) This option shows "skips" across the cascade in each chart, as indicated by the y intercept. If FALSE, each stage will start only with people who have not moved on to a subsequent stage, i.e. the y intercept will always be 0. If TRUE, an individual can enter into a stage even if they have "skipped" it. For example, an individual may go straight from stage 1 to stage 3, skipping 2. If this indicator is FALSE, the stage transition chart from 2-3 will not contain this individual in the denomenator. If TRUE, this individual will be counted in the denomenator for this transition, but will be counted as having transitioned into stage 3 immediately upon entering stage 2.
 #' @param x.axis.max This option shows the maximum range of the x axis in days. Defaults to 365 days (1 year).
 #' @param nochart Setting this to TRUE prevents the function from generating the main chart.
+#' @param risk.pool.size.line Setting to TRUE adds an indicator of risk pool remaining to the main charts as a line reflected beneath the main chart, showing the proportion of the original risk pool remaining at each time point. Defaults to FALSE.
 #' @param main.fill.colors (optional) This defines the color scheme of the stage transition graphs, as a string indicator for color or a c() list of colors. If the colors contain only one color, the color scheme will automatically generate progressively faded versions of the initial color provided for the remaining stage transitions. Otherwise, a list which is exactly one fewer than the # of stages must be provided, in the order of stage trasitions.
 #' @param death.fill.color (optional) This defines the color scheme for the death stage transition, as a string indicator for color.
+#' @param risk.pool.fill.color (optional) This defines the color scheme for the risk pool graphic, as a string indicator for color.
 #' @import survival ggplot2 dplyr tidyr zoo scales grDevices
 #' @importFrom stats relevel
 #' @importFrom rlang .data
@@ -27,7 +29,7 @@
 #' @examples
 #' # Pull in data from example simulated dataset
 #' library(longitudinalcascade)
-#' data(events_long_cascade_sim)
+#' data(events_long)
 #'
 #' # Set up options
 #' stages.order <- c("First tested positive","Knows status","Linked to care","Eligible for ART",
@@ -61,8 +63,9 @@ longitudinalcascade <- function(events.long,stages.order,groups.order=NA,
                          allow.sub.lines=FALSE,allow.skips=FALSE,
                          groups.date.breaks=NA,
                          x.axis.max=365,
-                         main.fill.colors = "#4472C4",death.fill.color = "indianred1",
-                         nochart=FALSE) {
+                         main.fill.colors = "#4472C4",death.fill.color = "#FF6A6A",
+                         nochart=FALSE,risk.pool.size.line=FALSE,
+                         risk.pool.fill.color = "#90dbb2") {
   # Data manipulation
   {
     # Transform data into wide form
@@ -90,28 +93,28 @@ longitudinalcascade <- function(events.long,stages.order,groups.order=NA,
         names(events.wide) <- gsub("stage.", "date.stage.",names(events.wide))
     }
     # Generate time-based groups if time breaks are specified
-      if (anyNA(groups.date.breaks)==FALSE){
-        # Generate group names from breaks
-          groups.order <- c(paste0(as.character(groups.date.breaks[1])," to ",as.character(groups.date.breaks[2]-1)))
-          for (i in 2:(length(groups.date.breaks)-1)){
-            groups.order <- c(groups.order,paste0(as.character(groups.date.breaks[i])," to ",as.character(groups.date.breaks[i+1]-1)))
-          }
-        # Generate grouping for each event except the last to determine whether the start event is within time breaks
-          for (stage.index in 1:(length(stages.order)-1)){
-            events.wide[[paste0("date.stage.",stage.index,".group")]] <- NA
-            for (break.index in 1:(length(groups.date.breaks)-1)){
-              events.wide[[paste0("date.stage.",stage.index,".group")]] <- ifelse(
-                (events.wide[[paste0("date.stage.",stage.index)]] >= groups.date.breaks[break.index]) & (events.wide[[paste0("date.stage.",stage.index)]] < groups.date.breaks[break.index+1]),
-                groups.order[break.index],
-                events.wide[[paste0("date.stage.",stage.index,".group")]])
-                #events.wide[[paste0("date.stage.group.",group.index)]])
-            }
+    if (anyNA(groups.date.breaks)==FALSE){
+      # Generate group names from breaks
+        groups.order <- c(paste0(as.character(groups.date.breaks[1])," to ",as.character(groups.date.breaks[2]-1)))
+        for (i in 2:(length(groups.date.breaks)-1)){
+          groups.order <- c(groups.order,paste0(as.character(groups.date.breaks[i])," to ",as.character(groups.date.breaks[i+1]-1)))
+        }
+      # Generate grouping for each event except the last to determine whether the start event is within time breaks
+        for (stage.index in 1:(length(stages.order)-1)){
+          events.wide[[paste0("date.stage.",stage.index,".group")]] <- NA
+          for (break.index in 1:(length(groups.date.breaks)-1)){
             events.wide[[paste0("date.stage.",stage.index,".group")]] <- ifelse(
-              is.na(events.wide[[paste0("date.stage.",stage.index,".group")]]),
-              "No group membership",
+              (events.wide[[paste0("date.stage.",stage.index)]] >= groups.date.breaks[break.index]) & (events.wide[[paste0("date.stage.",stage.index)]] < groups.date.breaks[break.index+1]),
+              groups.order[break.index],
               events.wide[[paste0("date.stage.",stage.index,".group")]])
+              #events.wide[[paste0("date.stage.group.",group.index)]])
           }
-      } else{}
+          events.wide[[paste0("date.stage.",stage.index,".group")]] <- ifelse(
+            is.na(events.wide[[paste0("date.stage.",stage.index,".group")]]),
+            "No group membership",
+            events.wide[[paste0("date.stage.",stage.index,".group")]])
+        }
+    } else{}
     # Fix ordering, so that completion of a later stage indicates completion of earlier stage, and mark where this happens
     {
       for (stage.index in seq(length(stages.order)-1,1,-1)){
@@ -292,12 +295,14 @@ longitudinalcascade <- function(events.long,stages.order,groups.order=NA,
           # Generate survival data
             chart.surv <- survival::survfit(survival::Surv(time = chart.time, event = chart.event) ~ 1)
             surv.time <- chart.surv$time
-            surv.surv <- 1-chart.surv$surv
-            surv.surv.UB <- 1-chart.surv$lower
-            surv.surv.LB <- 1-chart.surv$upper
-            surv.n <- chart.surv$n
-            surv.data <- data.frame(surv.time,surv.surv,surv.surv.UB,surv.surv.LB,surv.n)
-            #surv.data <- data.frame(surv.time,surv.surv)
+            surv.p <- 1-chart.surv$surv
+            surv.p.UB <- 1-chart.surv$lower
+            surv.p.LB <- 1-chart.surv$upper
+            surv.n.t0 <- chart.surv$n
+            surv.n.atrisk <- chart.surv$n.risk
+            surv.p.atrisk <- chart.surv$n.risk/surv.n.t0
+            surv.data <- data.frame(surv.time,surv.p,surv.p.UB,surv.p.LB,surv.n.t0,surv.n.atrisk,surv.p.atrisk)
+            #surv.data <- data.frame(surv.time,surv.p)
             surv.data$start.stage.index <- start.stage.index
             surv.data$end.stage.index <- end.stage.index
             surv.data$group.index <- group.index
@@ -346,8 +351,8 @@ longitudinalcascade <- function(events.long,stages.order,groups.order=NA,
           # Generate survival data
             chart.surv <- survival::survfit(survival::Surv(time = chart.time, event = chart.event) ~ 1)
             surv.time <- chart.surv$time
-            surv.surv <- 1-chart.surv$surv
-            surv.data <- data.frame(surv.time,surv.surv)
+            surv.p <- 1-chart.surv$surv
+            surv.data <- data.frame(surv.time,surv.p)
             surv.data$start.stage.index <- start.stage.index
             surv.data$end.stage.index <- end.stage.index
             surv.data$group.index <- group.index
@@ -373,6 +378,7 @@ longitudinalcascade <- function(events.long,stages.order,groups.order=NA,
       }
     }
   }
+
   # Generate chart graphics
   {
     if (nochart==FALSE){
@@ -407,7 +413,60 @@ longitudinalcascade <- function(events.long,stages.order,groups.order=NA,
           if (length(main.fill.colors)==1){
             main.fill.colors <- color.gradient(main.fill.colors,(length(stages.order)-1))
           } else {}
-          
+
+      }
+      # Graphical functions
+      {
+        # Step ribbon function. Note: Original author of code was Triad sou from the RcmdrPlugin.KMggplot2 package
+        {
+          geom_stepribbon <- function(
+            mapping = NULL, data = NULL, stat = "identity", position = "identity",
+            na.rm = FALSE, show.legend = NA, inherit.aes = TRUE, kmplot = FALSE, ...) {
+            layer(
+              data = data,
+              mapping = mapping,
+              stat = stat,
+              geom = GeomStepribbon,
+              position = position,
+              show.legend = show.legend,
+              inherit.aes = inherit.aes,
+              params = list(
+                na.rm = na.rm,
+                kmplot = kmplot,
+                ...
+              )
+            )
+          }
+          GeomStepribbon <- ggproto(
+            "GeomStepribbon", GeomRibbon,
+            extra_params = c("na.rm", "kmplot"),
+            draw_group = function(data, panel_scales, coord, na.rm = FALSE) {
+              #if (na.rm) data <- data[complete.cases(data[c("x", "ymin", "ymax")]), ]
+              data <- rbind(data, data)
+              data <- data[order(data$x), ]
+              data$x <- c(data$x[2:nrow(data)], NA)
+              #data <- data[complete.cases(data["x"]), ]
+              GeomRibbon$draw_group(data, panel_scales, coord, na.rm = FALSE)
+            },
+            setup_data = function(data, params) {
+              if (params$kmplot) {
+                data <- data[order(data$PANEL, data$group, data$x), ]
+                tmpmin <- tmpmax <- NA
+                for (i in 1:nrow(data)) {
+                  if (is.na(data$ymin[i])) {
+                    data$ymin[i] <- tmpmin
+                  }
+                  if (is.na(data$ymax[i])) {
+                    data$ymax[i] <- tmpmax
+                  }
+                  tmpmin <- data$ymin[i]
+                  tmpmax <- data$ymax[i]
+                }
+              }
+              data
+            }
+          )
+        }
       }
       # Data manipulation
       {
@@ -420,44 +479,41 @@ longitudinalcascade <- function(events.long,stages.order,groups.order=NA,
           # Drop out duplicate boxes (due to censoring) to reduce drawing time
             surv.combined.chart <- surv.combined.chart %>%
               dplyr::arrange(.data$group.factor,.data$start.stage.factor,.data$end.stage.factor,.data$surv.time) %>%
-              dplyr::group_by(.data$surv.surv,.data$start.stage.factor,.data$end.stage.factor,.data$group.factor) %>%
+              dplyr::group_by(.data$surv.p,.data$start.stage.factor,.data$end.stage.factor,.data$group.factor) %>%
               dplyr::slice(c(n()))
           # Generate beginning axis events to keep fill graphics going from start of chart at 0
             surv.combined.chart.beginning <- surv.combined.chart
-            surv.combined.chart.beginning$surv.surv <- 0
+            surv.combined.chart.beginning$surv.p <- 0
             surv.combined.chart.beginning$surv.time <- -1
-            surv.combined.chart.beginning$surv.surv.LB <- 0
-            surv.combined.chart.beginning$surv.surv.UB <- 0
+            surv.combined.chart.beginning$surv.p.LB <- 0
+            surv.combined.chart.beginning$surv.p.UB <- 0
             surv.combined.chart.beginning$surv.n <- 0
+            surv.combined.chart.beginning$surv.p.atrisk <- 1
             surv.combined.chart.beginning <- unique(surv.combined.chart.beginning)
             surv.combined.chart.beginning <- surv.combined.chart.beginning[, colnames(surv.combined.chart)]
             surv.combined.chart <- rbind(surv.combined.chart,surv.combined.chart.beginning)
             # Rearrange and sort for drawing
               surv.combined.chart <- surv.combined.chart %>%
                 dplyr::arrange(.data$group.factor,.data$start.stage.factor,.data$end.stage.factor,.data$surv.time)
-          # Generate end of exis events to keep fill graphics going to end of chart
+          # Generate end of axis events to keep fill graphics going to end of chart
             surv.combined.chart.end <- surv.combined.chart %>%
               dplyr::group_by(.data$start.stage.index,.data$end.stage.index,.data$group.index) %>%
-              dplyr::slice(which.max(.data$surv.surv))
+              dplyr::slice(which.max(.data$surv.p))
             surv.combined.chart.end$surv.time <- x.axis.max + 1
             surv.combined.chart <- rbind(surv.combined.chart,surv.combined.chart.end)
-            surv.combined.chart.end$surv.surv <- 0
+            surv.combined.chart.end$surv.p <- 0
+            surv.combined.chart.end$surv.p.atrisk <- 1
             surv.combined.chart <- rbind(surv.combined.chart,surv.combined.chart.end)
-          # Generate rectangle ends for drawing shading
-            #surv.combined.chart$rect.end <- ifelse(lead(surv.combined.chart$surv.surv)==0,x.axis.max,lead(surv.combined.chart$surv.surv))
-          # Generate end of axis events to keep graphics going until the end of chart
-            # surv.combined.chart.extra <- surv.combined.chart
-            # surv.combined.chart.extra$surv.surv <- 0
-            # surv.combined.chart.extra$surv.time <- x.axis.max
-            # surv.combined.chart.extra <- unique(surv.combined.chart.extra)
-            # surv.combined.chart.extra <- surv.combined.chart.extra[, colnames(surv.combined.chart)]
-            # surv.combined.chart <- rbind(surv.combined.chart,surv.combined.chart.extra)
           # Rearrange and sort for drawing
             surv.combined.chart <- surv.combined.chart %>%
               dplyr::arrange(.data$group.factor,.data$start.stage.factor,.data$end.stage.factor,.data$surv.time)
           # Temporary for putting in years
             surv.combined.chart$surv.time = surv.combined.chart$surv.time/365
-
+          # Generate risk pool dataset
+            surv.combined.chart.risk.pool <- surv.combined.chart[surv.combined.chart$end.stage.index == (surv.combined.chart$start.stage.index +1) ,]
+          # Add a dummy factor for legend
+            surv.combined.chart.risk.pool$event.factor <- factor("Risk pool",labels = c("Risk pool"),levels = c("Risk pool"))
+          # Generate version which includes end steps for stepwise fill
         }
         # Death events
         {
@@ -469,11 +525,11 @@ longitudinalcascade <- function(events.long,stages.order,groups.order=NA,
             # Drop out duplicate boxes (due to censoring) to reduce drawing time
               surv.death.combined.chart <- surv.death.combined.chart %>%
                 dplyr::arrange(.data$group.factor,.data$start.stage.factor,.data$end.stage.factor,.data$surv.time) %>%
-                dplyr::group_by(.data$surv.surv,.data$start.stage.factor,.data$end.stage.factor,.data$group.factor) %>%
+                dplyr::group_by(.data$surv.p,.data$start.stage.factor,.data$end.stage.factor,.data$group.factor) %>%
                 dplyr::slice(c(n()))
             # Generate beginning of axis events to keep graphics going until the end of chart
               surv.death.combined.chart.extra <- surv.death.combined.chart
-              surv.death.combined.chart.extra$surv.surv <- 0
+              surv.death.combined.chart.extra$surv.p <- 0
               surv.death.combined.chart.extra$surv.time <- 0
               surv.death.combined.chart.extra <- unique(surv.death.combined.chart.extra)
               surv.death.combined.chart.extra <- surv.death.combined.chart.extra[, colnames(surv.death.combined.chart)]
@@ -483,7 +539,7 @@ longitudinalcascade <- function(events.long,stages.order,groups.order=NA,
                 dplyr::arrange(.data$group.factor,.data$start.stage.factor,.data$end.stage.factor,.data$surv.time)
             # Generate end of axis events to keep graphics going until the end of chart
               surv.death.combined.chart.extra <- surv.death.combined.chart
-              surv.death.combined.chart.extra$surv.surv <- 0
+              surv.death.combined.chart.extra$surv.p <- 0
               surv.death.combined.chart.extra$surv.time <- x.axis.max
               surv.death.combined.chart.extra <- unique(surv.death.combined.chart.extra)
               surv.death.combined.chart.extra <- surv.death.combined.chart.extra[, colnames(surv.death.combined.chart)]
@@ -493,17 +549,18 @@ longitudinalcascade <- function(events.long,stages.order,groups.order=NA,
                 dplyr::arrange(.data$group.factor,.data$start.stage.factor,.data$end.stage.factor,.data$surv.time)
             # Temporary for putting in years
               surv.death.combined.chart$surv.time = surv.death.combined.chart$surv.time/365
+            # Add dummy event factor for legends
+              surv.death.combined.chart$event.factor <- factor(death.indicator,labels = c(death.indicator),levels = c(death.indicator))
           }
         }
       }
       # Generate chart
       {
-        chart <- ggplot2::ggplot(data=surv.combined.chart) +
-          #geom_rect(aes(xmin=surv.time,xmax=lead(surv.time),ymin=0,ymax=surv.surv,fill=end.stage.factor),alpha=1) +
-          #geom_rect(aes(xmin=surv.time,xmax=surv.time,ymin=0,ymax=surv.surv,fill=end.stage.factor),alpha=1) +
-          ggplot2::geom_polygon(aes(x=.data$surv.time,y=.data$surv.surv,fill=.data$end.stage.factor),alpha=1) +
-          ggplot2::scale_fill_manual(values=main.fill.colors) +
-          ggplot2::geom_step(aes(x=.data$surv.time,y=.data$surv.surv,color=.data$end.stage.factor)) +
+        chart <- ggplot2::ggplot() +
+          geom_stepribbon( data=surv.combined.chart,aes(x=.data$surv.time,ymax=.data$surv.p,fill=.data$end.stage.factor,ymin=0),
+                                                 alpha=1) +
+          ggplot2::geom_step(data=surv.combined.chart,aes(x=.data$surv.time,y=.data$surv.p,color=.data$end.stage.factor),
+                             show.legend = FALSE) +
           ggplot2::theme_bw() %+replace%
           ggplot2::theme(
             panel.grid = element_blank(),
@@ -512,13 +569,14 @@ longitudinalcascade <- function(events.long,stages.order,groups.order=NA,
             legend.position="bottom",
             legend.title=element_blank(),
             axis.text = element_text(colour="black",size=10),
-            strip.text = element_text(size = 12,hjust=0),
+            strip.text = element_text(size = 12),
+            strip.text.x = element_text(hjust=0),
             panel.spacing = unit(1, "lines")
           ) +
+          ggplot2::guides(color = guide_legend(override.aes = list(linetype = 0))) +
           ggplot2::scale_x_continuous(expand = c(0, 0),
                             labels=x.scale.function,
                             breaks = c(0,round2(x.axis.range/365,0))) +
-          ggplot2::coord_cartesian(xlim=c(0,(x.axis.range/365)),ylim = c(0, 1)) +
           ggplot2::xlab("Time (years) from start of stage") +
           ggplot2::scale_y_continuous(expand = c(0, 0),labels=percent) +
           ggplot2::scale_color_manual(values=c(rep("black",length(stages.order)-1))) +
@@ -526,14 +584,33 @@ longitudinalcascade <- function(events.long,stages.order,groups.order=NA,
                      switch="y") +
           ggplot2::theme(strip.background = element_blank(),
                 strip.placement = "outside")
+      # Add risk pool proportion indicator if indicated
+        if (risk.pool.size.line==TRUE){
+          chart <- chart +
+            ggplot2::coord_cartesian(xlim=c(0,(x.axis.range/365)),ylim = c(-.2, 1)) +
+            geom_stepribbon(data = surv.combined.chart.risk.pool,
+                                  aes(x=.data$surv.time,ymin=((.data$surv.p.atrisk-1)/5),ymax=0),
+                                  alpha=1,fill=risk.pool.fill.color) +
+            ggplot2::geom_step(data = surv.combined.chart.risk.pool,aes(x=.data$surv.time,y=(.data$surv.p.atrisk-1)/5))
+        } else {
+          chart <- chart +
+            ggplot2::coord_cartesian(xlim=c(0,(x.axis.range/365)),ylim = c(0, 1))
+        }
       # Add death event if present
         if (is.na(death.indicator)==FALSE){
           chart <- chart +
-            ggplot2::geom_polygon(data=surv.death.combined.chart,aes(x=.data$surv.time,y=1-.data$surv.surv),alpha=1,fill=death.fill.color) + 
-            ggplot2::geom_step(data=surv.death.combined.chart,aes(x=.data$surv.time,y=1-.data$surv.surv))
-            #geom_rect(data=surv.death.combined.chart,aes(xmin=surv.time,xmax=lead(surv.time),ymin=1-surv.surv,ymax=1),alpha=1,fill=death.fill.color)
-            
-        }
+            geom_stepribbon(data=surv.death.combined.chart,aes(x=.data$surv.time,ymin=1-.data$surv.p,ymax=1),
+                                  alpha=1,fill=death.fill.color) +
+            #ggplot2::geom_polygon(data=surv.death.combined.chart,aes(x=.data$surv.time,y=1-.data$surv.p,fill=.data$event.factor),
+            #                      alpha=1) +
+            ggplot2::geom_step(data=surv.death.combined.chart,aes(x=.data$surv.time,y=1-.data$surv.p))
+        } else {}
+      # Add touchup graphics
+        chart <- chart +
+          ggplot2::geom_hline(yintercept=0)
+      # Set fill colors
+        chart <- chart +
+          ggplot2::scale_fill_manual(values = main.fill.colors)
       }
     }
     else {
